#ifndef _HATPNODE_H
#define _HATPNODE_H

#include<iostream>
#include<sstream>
#include <string>
#include <vector>

static const std::string TYPE_ACTION = "action";
static const std::string TYPE_METATASK = "metatask";

/**
 * Representation of a node.
 * A node is the basis of the plan and represents a task (compound or primitive)
 */
class hatpNode {

	private:
		unsigned int nodeID;
		std::string nodeType;
		std::string taskName;
		std::vector<std::string> agents;
		unsigned int startTime;
		unsigned int endTime;
		std::vector<std::string> parameters;
		unsigned int rootnode;
		std::vector<unsigned int> subnodes;
		std::vector<unsigned int> streamPredecessors;
		std::vector<unsigned int> streamSuccessors;
		std::vector<unsigned int> treePredecessors;
		std::vector<unsigned int> treeSuccessors;
	
	public:

		/**
		 * Constructor of a node.
		 * @param id The ID of the node
		 * @param type The type of the node
		 * @param name The name of this task
		 * @param ags The list of agents
		 * @param stime The start time
		 * @param etime The end time
		 * @param params The list of parameters
		 */
		hatpNode(unsigned int id, std::string type, std::string name, std::vector<std::string> ags, unsigned int stime, unsigned int etime, 
				std::vector<std::string > params);

		/**
		 * Empty destructor.
		 */
		~hatpNode();

		/**
		 * Get the node ID.
		 * @return The ID of the current node
		 */
		unsigned int getID() const;

		/**
		 * Get the node type.
		 * @return The type of the current node
		 */
		std::string getType() const;

		/**
		 * 	Get the node name.
		 * @return The name of the current task represented by this node
		 */
		std::string getName() const;

		/**
		 * Get the agents list involved in the current task.
		 * @return a list of all the agents involved in this task
		 */
		std::vector<std::string> getAgents() const;

		/**
		 * Get the node start time.
		 * @return the time of the expected action start
		 */
		unsigned int getStartTime() const;

		/**
		 * Get the node end time.
		 * @return the time of the expected action end
		 */
		unsigned int getEndTime() const;

		/**
		 * Get the node parameters
		 * @return the list of parameters of the current task
		 */
		std::vector<std::string> getParameters() const;

		/**
		 * Get the root node.
		 * @return The ID of the root of the current node of 0 if it doesn't exist
		 */
		unsigned int getRootNode() const;

		/**
		 * Get the subnodes list.
		 * @return The list of subnodes of the current node
		 */
		std::vector<unsigned int> getSubNodes() const;

		/**
		 * Get the list of predecessors (in tree).
		 * @return the list of nodes predecessors of the current one in the tree
		 */
		std::vector<unsigned int> getTreePredecessors() const;

		/**
		 * Get the list of successors (in tree).
		 * @return the list of nodes successors of the current one in the tree
		 */
		std::vector<unsigned int> getTreeSuccessors() const;

		/**
		 * Get the list of predecessors (in streams).
		 * @return the list of nodes predecessors of the current one in the streams
		 */
		std::vector<unsigned int> getStreamPredecessors() const;

		/**
		 * Get the list of successors (in streams).
		 * @return the list of nodes successors of the current one in the streams
		 */
		std::vector<unsigned int> getStreamSuccessors() const;

		/**
		 * Inform if the node is related to an action task.
		 * @return True if the node is a primitive task (an action)
		 */
		bool isAction() const;

		/**
		 * Set the root node ID.
		 * @param n The root node ID
		 */
		void setRootNode(unsigned int n);

		/**
		 * Add a node the the subnodes list.
		 * @param n the ID of the node to add to the subnodes list
		 */
		void addSubNode(unsigned int n);

		/**
		 * Add a predecessor node to the current node (in the tree).
		 * @param n The ID of the node to add to the Tree Predecessors list
		 */
		void addTreePredecessor(unsigned int n);

		/**
		 * Add a successor ndoe to the current node (in the tree).
		 * @param n The ID of the node to add to the Tree Successors list
		 */
		void addTreeSuccessor(unsigned int n);

		/**
		 * add a predecessor to the current node (in the streams).
		 * @param n The ID of the node to add to Streams Predecessors list
		 */
		void addStreamPredecessor(unsigned int n);

		/**
		 * add a successor to the current node (in the streams).
		 * @param n The ID of the node to add to the Streams Successors list 
		 */
		void addStreamSuccessor(unsigned int n);

		/**
		 * Get the node under a text form (for printing).
		 * @return A string containing the node under a text form
		 */
		std::string toString() const;

};

#endif
