#ifndef _HATPTREE_H
#define _HATPTREE_H

#include<map>

#include "hatpNode.hh"
#include "hatpTaskList.hh"

/**
 * Representation of a plan under a plan tree form.
 * A tree is a set of nodes linked by causal and hierarchical links.
 */
class hatpTree {

	private:
		std::map<unsigned int, hatpNode*> nodes;
		unsigned int root;
	
	protected:

		/**
		 * Used by getTreeAsTaskLists() to get the tree under a set of tasklists form
		 * @see getTreeAsTaskLists()
		 * @param f a internal flag to differ from getTreeAsTaskLists()
		 * @return The tree turned into a set of tasklists
		 */
		std::vector<hatpTaskList*> getTreeAsTaskLists(bool f) const;
	
	public:

		/**
		 * Constructor of a plan tree.
		 * @param r The root node ID of the tree
		 * @param n The list of all nodes contained in the plan tree
		 */
		hatpTree(unsigned int r, std::vector<hatpNode*> n);

		/**
		 * Empty destructor
		 */
		~hatpTree();

		/**
		 * Get the root node ID of this plan tree.
		 * @see getRootNode() to get the root node as a pointer
		 * @return The root node ID of the plan tree
		 */
		unsigned int getRootID() const;

		/**
		 * Get a pointer to the root node of this plan tree.
		 * @see getRootID() to get the root node ID
		 * @return A pointer to the root node of the plan tree
		 */
		hatpNode* getRootNode() const;

		/**
		 * Get all nodes of the plan tree.
		 * @return The list of all the nodes contained in the plan tree
		 */
		std::vector<hatpNode*> getNodes() const;

		/**
		 * Get a subtree starting from the node r.
		 * @param r The root node of the subtree to obtain
		 * @return A new tree with r as root node
		 */
		hatpTree* getSubTree(hatpNode* r) const;

		/**
		 * Get the plan tree depth.
		 * @return The depth of the plan tree
		 */
		unsigned int getTreeDepth() const;

		/**
		 * Get all the node at a given level.
		 * @param level The level for which the nodes are requested
		 * @return The list of nodes of the given level
		 */
		std::vector<hatpNode*> getNodesAtLevel(unsigned int level) const;

		/**
		 * Get the root of the plan tree under the form of a taskList.
		 * @return a taskList containing the root of the tree
		 */
		hatpTaskList* getRootAsTaskList() const;

		/**
		 * Get direct subnodes of the root as a TaskList.
		 * @return A TaskList containing the subnodes of the tree root
		 */
		hatpTaskList* getSubNodesAsTaskList() const;

		/**
		 * Get the tree as a set of TaskList.
		 * @return A list of TaskList containing all nodes under the form of TaskLists
		 */
		std::vector<hatpTaskList*> getTreeAsTaskLists() const;

		/**
		 * Get a set of TaskLists containing nodes of a givent level
		 * @param level The level from which nodes are required
		 * @return A list of TaskLists containing nodes of a given tree level
		 */
		std::vector<hatpTaskList*> getTaskListsAtLevel(unsigned int level) const;

		/**
		 * Get the causal links of the current plan tree.
		 * @return The list of causal links under the form of pairs of nodes ID	
		 */
		std::vector<std::pair<unsigned int, unsigned int> > getTreeCausalLinks() const;

		/**
		 * Get the hierarchical links of the current plan tree.
		 * @return The list of hierarchical links under the form of pairs of nodes ID	
		 */
		std::vector<std::pair<unsigned int, unsigned int> > getTreeHierarchicalLinks() const;

		/**
		 * Get the plan tree under a text form.
		 * @return A string representing the plan tree
		 */
		std::string toString() const;
		
};

#endif
